/*
 * @(#)MakeDoc.java (V0.2 --2000/08/25)
 *
 * Copyright (c) 1999-2000 Takezou. <takeshi_kanno@est.hi-ho.ne.jp>
 *
 * Riko PDF Maker 
 *
 * #    Comment  
 * T    Draw text
 * P    New section
 * R    Line feed
 * Hn   Draw header
 * M    Move current low
 * L    Draw line
 * BH   Draw bold font text(header font)
 * BT   Draw bold font text(default font)
 * r    Draw small font text with right alignment
 *
 * Command syntax is
 *  java jp.gr.java_conf.riko.examples.MakeDoc <DocFileName> 
 *
 */

package jp.gr.java_conf.riko.examples;

import jp.gr.java_conf.riko.*;
import jp.gr.java_conf.riko.font.*;
import java.io.*;
import java.util.*;

public class MakeDoc{
    private final String DEFAULT_SOURCE = "doc.txt";
    private final int SEPALATE_POS = 160;
    private final int VMARGIN = 70;
    private final int HMARGIN = 65;
    private final int DEFAULT_LEADING = 12;
    private final int DEFAULT_FONT_SIZE = 10;
    private final int HEADER_FONT1 = 14;
    private final int HEADER_FONT2 = 11;
    private final int TITLE_FONT = 22;
    private final int SMALL_FONT_SIZE = 8;
    private int currentX;
    private int currentY;
    private int maxHeight;
    private int numOfPrintable;
    private BufferedReader reader;
    private StringBuffer buf;
    private PdfProducer pp;
    private int[] saveIndex; 
    private Vector saveEntry;
    private String filename;
    private int currentSepalatePos;

    public MakeDoc(){
        super();
        pp = new PdfProducer();
        saveIndex = new int[10];
        saveEntry = new Vector(10);
    }

    private void clearIndexArray(int level){
        for (int i=level;i<saveIndex.length;i++){
            saveIndex[i] = 0;
        }
    }

    private void fileOpen(String fileName) throws FileNotFoundException{
        if (fileName != null) {
            reader = new BufferedReader(new FileReader(fileName));
        } else {
            reader = new BufferedReader(new FileReader(DEFAULT_SOURCE));
        }
    }

    public void makeDocument(String fileName) throws IOException {
        fileOpen(fileName);
        this.filename = fileName;
        currentX = currentSepalatePos;
        currentY = pp.getPageHeight() - VMARGIN - DEFAULT_LEADING;
        currentSepalatePos = SEPALATE_POS;
        buf = new StringBuffer();
        pp.setCreator("MakeDoc : Riko PDF Producer");
        pp.beginDoc(fileName + ".pdf");
        String s = reader.readLine();
        if (s.charAt(0) != 't') {
            System.out.println("Document source must start 't' character");
        } else {
            pp.getCanvas().setFont("jp.gr.java_conf.riko.font.GothicBold", TITLE_FONT);
            int w = new Double(pp.getCanvas().getTextWidth(s.substring(2))).intValue();
            int x = pp.getPageWidth() - w - HMARGIN;
            int y = pp.getPageHeight() / 3 * 2;
            pp.getCanvas().textOut(x, y, s.substring(2));
            pp.setTitle(s.substring(2));
            currentY = y;
            while (processLine() == true);
            flushBuffer();
        }
        pp.endDoc();
    }

    private boolean processLine() throws IOException{
        String s = reader.readLine();
        if (s == null) {
            return false;
        }
        if (s.length() < 1) {
            return true;
        }
        char ch = s.charAt(0);
        if (ch == 'T') {
            buf.append(s.substring(2));
        } else 
        if (ch == 'P') {
            flushBuffer();
            if (currentY != pp.getPageHeight() - VMARGIN - DEFAULT_LEADING){
                currentY = currentY - DEFAULT_LEADING;
            }
            if (s.length() >= 5) {
              int i = new Integer(s.substring(2,5)).intValue();
              currentSepalatePos = i;
            } else {
              currentSepalatePos = SEPALATE_POS;
            }
        } else
        if (ch == 'R') {
            flushBuffer();
            if (s.length() >= 5) {
              int i = new Integer(s.substring(2,5)).intValue();
              currentSepalatePos = i;
            } else {
              currentSepalatePos = SEPALATE_POS;
            }
        } else
        if (ch == 'H') {
            int i = new Integer(s.substring(1,2)).intValue();
            clearIndexArray(i+1);
            saveIndex[i] = saveIndex[i] + 1;
            flushBuffer();
            writeHeader(s.substring(3), i);
        } else
        if (ch == 'L') {
            currentY = currentY + DEFAULT_LEADING - 2;
            double d = new Double(s.substring(2,5)).doubleValue();
            pp.getCanvas().drawLine(HMARGIN, currentY, 
                              pp.getPageWidth() - VMARGIN, currentY, d);
            currentY = currentY - DEFAULT_LEADING - 2;
        } else
        if (ch == 'M') {
            int i = new Integer(s.substring(2,5)).intValue();
            currentY = currentY - i;
        } else
        if (ch == 'r') {
            flushBuffer();
            pp.getCanvas().beginText();
            pp.getCanvas().setFont("jp.gr.java_conf.riko.font.Gothic", SMALL_FONT_SIZE);
            int w = new Double(pp.getCanvas().getTextWidth(s.substring(2))).intValue();   
            currentY = currentY - 10;
            pp.getCanvas().textOut(pp.getPageWidth() - HMARGIN - w,
                                         currentY, s.substring(2));
            pp.getCanvas().beginText();
        } else
        if (ch == 'B') {
            flushBuffer();
            pp.getCanvas().beginText();
            if (s.charAt(1) == 'H') {
              pp.getCanvas().setFont("jp.gr.java_conf.riko.font.GothicBold", HEADER_FONT2);
            } else {
              pp.getCanvas().setFont("jp.gr.java_conf.riko.font.GothicBold", DEFAULT_FONT_SIZE);
            }
            pp.getCanvas().textOut(currentSepalatePos, currentY, s.substring(3));
            pp.getCanvas().beginText();
            currentY = currentY - DEFAULT_LEADING;
        }

        return true;
    }

    private void writeHeader(String title, int level){
        OutlineEntry pe = null;
        if (level == 0 && currentY != pp.getPageHeight() - VMARGIN - DEFAULT_LEADING){
            pp.newPage();
            writeTop();
            currentY = pp.getPageHeight() - VMARGIN - DEFAULT_LEADING;
        }
        StringBuffer ht = new StringBuffer();
        for (int i=0;i<level+1;i++){
            ht.append(new Integer(saveIndex[i]).toString());
            ht.append(". ");
        }
        boolean isOpened = false;
        if (level < 1) {
          //  isOpened = true;
            pp.getCanvas().setFont("jp.gr.java_conf.riko.font.GothicBold", HEADER_FONT1);
            currentY = currentY + DEFAULT_LEADING - HEADER_FONT1 - 10;             
        } else {
            pe = (OutlineEntry)saveEntry.elementAt(level-1);
            pp.getCanvas().setFont("jp.gr.java_conf.riko.font.GothicBold", HEADER_FONT2);
            currentY = currentY + DEFAULT_LEADING - HEADER_FONT2 - 10; 
        } 
        int w = new Double(pp.getCanvas().getTextWidth(ht.toString())).intValue();
        ht.append(title);
        pp.getCanvas().textOut(currentSepalatePos - w, currentY, ht.toString());
        OutlineEntry e = pp.getOutlines().addEntry(pe, ht.toString(), 
                              currentSepalatePos - w, currentY+24, isOpened);
        if (level < saveEntry.size()) {
            saveEntry.setElementAt(e, level);
        } else { 
            saveEntry.addElement(e);
        }
        if (level == 0) {
            currentY = currentY - 10;
            pp.getCanvas().drawLine(HMARGIN, currentY, 
                                        pp.getPageWidth() - VMARGIN, currentY, 2);
            currentY = currentY - 2;
        }
        currentY = currentY - DEFAULT_LEADING - 4;
    }

    private void writeTop(){
        String s = new String(filename);
        pp.getCanvas().beginText();
        pp.getCanvas().setFont("jp.gr.java_conf.riko.font.Gothic", 8);
        int w = new Double(pp.getCanvas().getTextWidth(s)).intValue();   
        pp.getCanvas().textOut(pp.getPageWidth() - HMARGIN - w,
                                         pp.getPageHeight() - VMARGIN + 10, s);
        pp.getCanvas().endText();
    }

    private void changePage(){
        pp.getCanvas().endText();
        pp.newPage();
        currentY = pp.getPageHeight() - VMARGIN - DEFAULT_LEADING;
        writeTop();
        beginText();
    }

    private void beginText(){
        pp.getCanvas().beginText();
        pp.getCanvas().setLeading(DEFAULT_LEADING);
        pp.getCanvas().setFont("jp.gr.java_conf.riko.font.Gothic", DEFAULT_FONT_SIZE);
        currentX = currentSepalatePos;
        pp.getCanvas().moveTextPoint(currentX, currentY);
        maxHeight = currentY - VMARGIN + DEFAULT_LEADING;
        numOfPrintable = maxHeight / DEFAULT_LEADING;
    }

    private void flushBuffer(){
        if (buf.toString().length() == 0) {
            return;
        }
        beginText();
        String s = pp.getCanvas().formatText(buf.toString(),
                           pp.getPageWidth() - HMARGIN - currentSepalatePos);
        Vector v = pp.getCanvas().splitText(s);
        if (numOfPrintable < 1) {
            changePage();
        }

        while (true) {
            if (v.size() <= numOfPrintable) {
                currentY = currentY - v.size() * DEFAULT_LEADING;
                pp.getCanvas().showTextEx(v);
                break;
            } else {
                Vector v2 = new Vector();
                for (int i=0; i<numOfPrintable; i++){
                    v2.addElement(v.elementAt(0));
                    v.removeElementAt(0);
                }
                currentY = currentY - v2.size() * DEFAULT_LEADING;
                pp.getCanvas().showTextEx(v2);
                changePage();
            }
        }       
        pp.getCanvas().endText();
        buf = new StringBuffer();
    }

    public static void main(String[] argv){
        try {
        if (argv.length < 0){
            System.out.println("Error no file specified..");
        }
        System.out.println(argv[0]);
            MakeDoc makeDoc = new MakeDoc();
            makeDoc.makeDocument(argv[0]);
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.exit(0);         
    }
}
